C**********************************************************************
C**********************************************************************
C** 
C**     File: kcd.for
C**
C**     Project: jitax_kcd
C**
C**     Description: Routines to read Bruker-Nonius kcd CCD
C**                  frame files.
C**
C**     WARNING: Because of lack of documentation this reader
C**              is very basic and will not necessarily work
C**              for other kcd files. Please feel free to hack
C**              away at the code to make it more general.
C**
C**********************************************************************
C**
C** Copyright (c) 2003 Scott A. Belmonte
C**
C** This file is part of jitax_kcd. Reproduction, in whole or in part,
C** is only permitted under licence. For licensing information see
C** licence.txt at the top of the jitax_kcd source tree.
C**
C** E-mail: scott.belmonte@ntlworld.com
C**
C**********************************************************************
C**********************************************************************
C
C**********************************************************************
C     Routine: OPEN_KCD_FILE
C
C     Description:
C        Opens the kcd file whose name is stored in NAME.
C
C        NB. Uses a fixed unit number. This means only one KCD
C        can be open at a time. CLOSE_KCD_FILE must be called
C        before OPEN_KCD_FILE can be called again.
C
C        Reads in header information and stores it in the COMMONS
C        found in kcd.inc
C
C        Returns: 0 file successfully opened, 
C                 1 error opening file
C**********************************************************************
      INTEGER FUNCTION OPEN_KCD_FILE(NAME)
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
C     Parameters
C
      CHARACTER*(*) NAME        ! TIFF file name
C
C     Functions
C
      LOGICAL BIGEND_CPU
      INTEGER READ_KCD_LINE
      INTEGER PARSE_KCD_INT
      REAL    PARSE_KCD_REAL
C
C     Variables
C
      LOGICAL   OPEN            ! .TRUE. if a file is already open
      CHARACTER LINE(MAXLINLEN) ! A line in the kcd
      INTEGER   LINELEN         ! Length in characters of the line
      INTEGER   OFFSET          ! Offset in the file to start reading
                                ! from.
C
C     Initialise the LAST_REC common. 0 means that BUFFER contains
C     no valid record.
C
      LAST_REC = 0
C
C     Check that a file isn't currently open. These routines
C     can only work with one file at a time.
C
      INQUIRE(UNIT=UNT, OPENED=OPEN)
      IF (OPEN) THEN
         WRITE(*,*) '** OPEN_KCD_FILE error: File already opened.'
         WRITE(*,*) '** Only one KCD file can be opened at a time.'
         GOTO 901
      ENDIF
C
C     Major assumption: KCD files are always little endian.
C     This may not be the case. Change BIG_END to .TRUE. 
C     to read files that are big endian.
C
      BIG_END = .FALSE.
C
C     If the machine has a different endian than the file
C     then SWAP is set to .TRUE.
C
      SWAP = (BIGEND_CPU() .XOR. BIG_END)
C
C     Open file
C
      OPEN(UNIT=UNT, FILE=NAME, STATUS='OLD', FORM='UNFORMATTED',
     $     ACCESS='DIRECT', RECL = REC_SIZE, ERR=901)
      REWIND(UNIT=UNT)
C
C     Read in each of the ASCII lines in the header and
C     convert to the appropriate value.
C
      OFFSET = 0
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 1, skip
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 2
      CALL PARSE_KCD_CHR(ID, LINE, LINELEN) 
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 3
      DETID = PARSE_KCD_INT(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 4
      GONID = PARSE_KCD_INT(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 5
      CALL PARSE_KCD_CHR(DATETIME, LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 6
      SLOT = PARSE_KCD_INT(LINE, LINELEN)      
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 7
      REPEATS = PARSE_KCD_INT(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 8
      READOUTS = PARSE_KCD_INT(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 9
      EXPTIME = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 10
      INTTIME = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 11
      WIDTH = PARSE_KCD_INT(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 12
      HEIGHT = PARSE_KCD_INT(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 13, skip
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 14
      THETA = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 15
      PHI = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 16
      PHIRANGE = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 17
      PHIINTERVAL = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 18
      OMEGA = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 19
      KAPPA = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 20
      DXSTART = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 21
      HVVOLT = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 22
      MACURRENT = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 23
      XSIZE = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 24
      YSIZE = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 25
      CALL PARSE_KCD_CHR(TARGET, LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 26
      ALPHA1 = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 27
      ALPHA2 = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 28
      ALPHRATIO = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 29
      CALL PARSE_KCD_CHR(POLARISATION, LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 30
      KAPPAANG = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 31
      SCINTYPE = PARSE_KCD_INT(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 32
      SCINLOAD = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 33
      CALL PARSE_KCD_CHR(SHUTTER, LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 34
      DETGAIN = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 35
      BEAMDIA = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 36
      BEAMDIST = PARSE_KCD_REAL(LINE, LINELEN)
      LINELEN = READ_KCD_LINE(LINE, OFFSET) ! Line 37
      CALL PARSE_KCD_CHR(DATTYPE, LINE, LINELEN)
      IF ((DATTYPE(1) .EQ. 'u') .AND.
     $    (DATTYPE(2) .EQ. '8')) THEN
         BITSPERSAMPLE = 8
      ELSE IF ((DATTYPE(1) .EQ. 'u') .AND.
     $         (DATTYPE(2) .EQ. '1') .AND.
     $         (DATTYPE(3) .EQ. '6')) THEN
         BITSPERSAMPLE = 16
      ELSE IF ((DATTYPE(1) .EQ. 'u') .AND.
     $         (DATTYPE(2) .EQ. '3') .AND.
     $         (DATTYPE(3) .EQ. '2')) THEN
         BITSPERSAMPLE = 32
      ELSE
         BITSPERSAMPLE = 0
      ENDIF
C         
      OPEN_KCD_FILE = 0
      RETURN
C
C     Error traps
C
 901  OPEN_KCD_FILE = 1        ! Error opening file
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_KCD_FILE
C
C     Description:
C        Reads bytes START_BYTE to END_BYTE, inclusive, from a kcd file 
C        into DATA.
C        NB. START_BYTE and END_BYTE are zero offset.
C     
C        Returns 0 if successful, 1 otherwise
C**********************************************************************
      INTEGER FUNCTION READ_KCD_FILE(DATA, START_BYTE, END_BYTE)
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
C     Parameters
C     
      BYTE    DATA(*)           ! The array to read the bytes into
      INTEGER START_BYTE        ! The byte to start reading from
      INTEGER END_BYTE          ! The byte to finish reading at.
C
C     Functions
C
      INTEGER READ_KCD_RECORD
C
C     Variables
C          
      INTEGER START_REC         ! Start record
      INTEGER END_REC           ! End record
      INTEGER TOTAL_BYTES       ! The total number of bytes to read
      INTEGER NEXT_BYTE         ! The next byte to write into DATA
      INTEGER CUR_REC           ! Current record being read
      INTEGER SKIP_BYTES        ! Num bytes to skip in first record
      INTEGER NUM_BYTES         ! Num bytes to read from first record
      INTEGER STATUS            ! Status of the read
      INTEGER I                 ! Loop counter
C     
C     Check for errors in the parameters START_BYTE and END_BYTE.
C     
      IF ((START_BYTE .GT. END_BYTE) .OR. (START_BYTE .LT. 0)) THEN
         WRITE(*,*) '** READ_KCD_FILE Error: Bad START_BYTE or
     $        END_BYTE'
         WRITE(*,*) '** START = ', START_BYTE, ', END = ', END_BYTE
         GOTO 911
      ENDIF
C     
C     Calculate the first and last records that have to be read, and
C     the total number of bytes that have to be read.
C     
      START_REC   = 1 + START_BYTE/REC_SIZE
      END_REC     = 1 + END_BYTE/REC_SIZE
      TOTAL_BYTES = END_BYTE - START_BYTE + 1
C     
      NEXT_BYTE = 1
      CUR_REC   = START_REC
C     
C     Read the first record (unless it already is in BUFFER).  
C     
      STATUS = 0
      IF(CUR_REC .NE. LAST_REC) THEN
         READ(UNIT=UNT, REC=CUR_REC, IOSTAT=STATUS) BUFFER
      ENDIF
C     
C     Copy the required bytes from BUFFER to DATA.
C     
      SKIP_BYTES = START_BYTE - (START_REC - 1)*REC_SIZE
      NUM_BYTES  = MIN(REC_SIZE - SKIP_BYTES, TOTAL_BYTES)
      DO I = 1, NUM_BYTES
         DATA(I) = BUFFER(I + SKIP_BYTES)
      ENDDO
      NEXT_BYTE = NEXT_BYTE + NUM_BYTES
      IF(STATUS .NE. 0) GOTO 911
C     
C     Read anymore whole records directly into DATA.
C     
      DO CUR_REC = START_REC + 1, END_REC - 1
         STATUS = READ_KCD_RECORD(DATA(NEXT_BYTE), CUR_REC)
         IF(STATUS .NE. 0) GOTO 911
         NEXT_BYTE = NEXT_BYTE + REC_SIZE
      ENDDO
C     
C     If needed, read the last record into BUFFER, then transfer to DATA.
C     
      CUR_REC = END_REC
      IF(CUR_REC .NE. START_REC) THEN
         READ(UNIT=UNT, REC=CUR_REC, IOSTAT=STATUS) BUFFER
         DO I = NEXT_BYTE, TOTAL_BYTES
            DATA(I) = BUFFER(I - NEXT_BYTE + 1)
         ENDDO
      ENDIF
C     
C     Store the record number that BUFFER contains, then return.
C     
      LAST_REC = CUR_REC
      READ_KCD_FILE = 0
      RETURN
C     
C     Error occured. Set LAST_REC = 0 so we don't reuse BUFFER contents.
C
C     Ignore errors that occur when reading END_REC since this
C     might be the last record in the file and may cause an error
C     because it is less than REC_SIZE in size.
C     
 911  IF (CUR_REC .EQ. END_REC) THEN
         LAST_REC = CUR_REC
         READ_KCD_FILE =0
      ELSE
         LAST_REC = 0
         READ_KCD_FILE = 1
      ENDIF
      RETURN
C     
      END
C
C
C
C**********************************************************************
C     Routine: READ_KCD_RECORD
C
C     Description:
C        Reads the record RECORD for open KCD file on unit UNT into
C        ARRAY.
C     
C        Returns 0 if successful, 1 otherwise
C**********************************************************************
      INTEGER FUNCTION READ_KCD_RECORD(ARRAY, RECORD)
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
C     Parameter
C
      BYTE    ARRAY(REC_SIZE)
      INTEGER RECORD
C
C     Variables
C
      INTEGER STATUS
C
      READ(UNIT=UNT, REC=RECORD, IOSTAT=STATUS) ARRAY
      READ_KCD_RECORD=STATUS
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_KCD_DATA
C
C     Description:
C        Reads the image data of frame FRAME into the DATA.
C
C        NB. DATA has to be big enough to contain all the image
C        data: i.e DATA(WIDTH*HEIGHT)
C
C        Each KCD file may contain one or more frames of data.
C        Frame numbers start at 1. The FRAME parameter gives the
C        number of the frame to read.
C
C        Returns 0 if successful, 1 if frame number less than 1,
C        2 if there is an error readind the file.
C
C**********************************************************************
      INTEGER FUNCTION READ_KCD_DATA(FRAME, IDATA)
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
C     Parameters
C
      INTEGER FRAME             ! The frame number to read
      INTEGER IDATA(*)           ! Array to contain the image data
C
C     Functions
C
      INTEGER READ_KCD_FILE
      INTEGER READ_KCD_SHORT
      INTEGER READ_KCD_LONG
C
C     Variables
C
      INTEGER    BLOCKSZ        ! Size of the RAWDATA buffer
      PARAMETER (BLOCKSZ=8192*4)
      INTEGER    I, J           ! Loop counters
      INTEGER    NUMPIX         ! Number of pixels
      INTEGER    BYTESPERPIX    ! Bytes per pixel
      INTEGER    PIXREAD        ! Number of pixels read so far
      INTEGER    OFFSET         ! Byte offset in file to read from
      BYTE       RAWDATA(BLOCKSZ)
      INTEGER    REMAINDER
C
      IF (FRAME .LT. 1) GOTO 921
      IF ((BITSPERSAMPLE .NE. 8) .AND.
     $    (BITSPERSAMPLE .NE. 16) .AND.
     $    (BITSPERSAMPLE .NE. 32)) GOTO 923
C     
C     Start at pixel zero. The data starts at byte 4100 in KCD files.
C     Read data in blocks of BLOCKSZ bytes.
C     
      NUMPIX      = WIDTH*HEIGHT
      BYTESPERPIX = BITSPERSAMPLE/8
      PIXREAD     = 0
      OFFSET  = 4100 + (FRAME-1)*NUMPIX*BYTESPERPIX
      DO I = 1, (NUMPIX*BYTESPERPIX/BLOCKSZ)
C
         IF (READ_KCD_FILE(RAWDATA, OFFSET, 
     $                     (OFFSET+BLOCKSZ-1)) .NE. 0) 
     $        GOTO 922
C
         DO J = 1, BLOCKSZ, BYTESPERPIX
            PIXREAD = PIXREAD + 1
            IF (BITSPERSAMPLE .EQ. 8) THEN
               IDATA(PIXREAD) = RAWDATA(J)
            ELSE IF (BITSPERSAMPLE .EQ. 16) THEN
               IDATA(PIXREAD) = READ_KCD_SHORT(RAWDATA(J))
            ELSE IF (BITSPERSAMPLE .EQ. 32) THEN
               IDATA(PIXREAD) = READ_KCD_LONG(RAWDATA(J))
            ENDIF
         ENDDO
         OFFSET = OFFSET + BLOCKSZ
      ENDDO
C
C     Read the remainder of the data if any
C
      REMAINDER = MOD((NUMPIX*BYTESPERPIX), BLOCKSZ)
      IF (REMAINDER .NE. 0) THEN
         IF (READ_KCD_FILE(RAWDATA, OFFSET, 
     $                     (OFFSET+REMAINDER-1)) .NE. 0) 
     $        GOTO 922
C
         DO J = 1, REMAINDER, BYTESPERPIX
            PIXREAD = PIXREAD + 1
            IF (BITSPERSAMPLE .EQ. 8) THEN
               IDATA(PIXREAD) = RAWDATA(J)
            ELSE IF (BITSPERSAMPLE .EQ. 16) THEN
               IDATA(PIXREAD) = READ_KCD_SHORT(RAWDATA(J))
            ELSE IF (BITSPERSAMPLE .EQ. 32) THEN
               IDATA(PIXREAD) = READ_KCD_LONG(RAWDATA(J))
            ENDIF
         ENDDO
      ENDIF
C
      READ_KCD_DATA = 0
      RETURN
C     
C     Error trap
C     
 921  READ_KCD_DATA = 1         ! Bad frame number
      RETURN
 922  READ_KCD_DATA = 2         ! Error reading file
      RETURN
 923  READ_KCD_DATA = 3         ! Bad BITSPERSAMPLE
      END
C
C
C
C**********************************************************************
C     Routine: CLOSE_KCD_FILE
C
C     Description:
C        Closes the KCD file.
C**********************************************************************
      SUBROUTINE CLOSE_KCD_FILE()
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
      CLOSE(UNIT=UNT)
      END
C
C     
C
C**********************************************************************
C     Routine: BIGEND_CPU
C
C     Description:
C        Returns .TRUE. if the machine is big endian
C**********************************************************************
      LOGICAL FUNCTION BIGEND_CPU()
      IMPLICIT NONE
C
C     Variables
C          
      BYTE         B(2)
      INTEGER*2    I2
      EQUIVALENCE (B, I2)
C
      I2 = 1
      BIGEND_CPU = (B(1) .NE. 1)
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_KCD_SHORT
C
C     Description:
C        Converts the 2 bytes in RAW to INTEGER*2 swapping
C        the bytes if necessary.
C**********************************************************************
      INTEGER FUNCTION READ_KCD_SHORT(RAW)
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
C     Parameters
C     
      BYTE         RAW(2)
C
C     Variables
C
      BYTE         B(2)
      INTEGER*2    I2
      EQUIVALENCE (B, I2)
C
      IF (SWAP) THEN
         B(1) = RAW(2)
         B(2) = RAW(1)
      ELSE
         B(1) = RAW(1)
         B(2) = RAW(2)
      ENDIF
C
C     The integer is unsigned so convert if necessary.
C
      IF (I2 .LT. 0) THEN
         READ_KCD_SHORT = 2**16 + I2
      ELSE
         READ_KCD_SHORT = I2
      ENDIF
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_KCD_LONG
C
C     Description:
C        Converts the 4 bytes in RAW to INTEGER*4 swapping
C        the bytes if necessary.
C**********************************************************************
      INTEGER FUNCTION READ_KCD_LONG(RAW)
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
C     Parameters
C     
      BYTE         RAW(4)
C
C     Variables
C          
      BYTE         B(4)
      INTEGER*4    I4
      EQUIVALENCE (B, I4)
C
      IF (SWAP) THEN
         B(1) = RAW(4)
         B(2) = RAW(3)
         B(3) = RAW(2)
         B(4) = RAW(1)
      ELSE
         B(1) = RAW(1)
         B(2) = RAW(2)
         B(3) = RAW(3)
         B(4) = RAW(4)
      ENDIF
C
      READ_KCD_LONG = I4
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: STRLEN
C
C     Description:
C        Returns the number of characters in STRING not including
C        non-printable characters.
C**********************************************************************
      INTEGER FUNCTION STRLEN(STRING)
      IMPLICIT NONE
      CHARACTER*(*) STRING
C
C     Returns the length of a string minus whitespace at the end.
C
      INTEGER I
C
      DO I = LEN(STRING), 1, -1
         IF (STRING(I:I) .GT. ' ') THEN
            GOTO 931
         ENDIF
      ENDDO
C
 931  STRLEN = I
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: READ_KCD_LINE
C
C     Description: 
C        Converts MAXLINLEN bytes starting from byte OFFSET into 
C        characters. The conversion stops if the line terminator
C        (10) is encountered.
C        
C**********************************************************************
C
      INTEGER FUNCTION READ_KCD_LINE(LINE, OFFSET)
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
C     Parameters
C
      CHARACTER*(*) LINE
      INTEGER       OFFSET
C
C     Functions
C
      INTEGER READ_KCD_FILE
      INTEGER STRLEN
C
C     Variables
C
      INTEGER I
      INTEGER LINELEN
      BYTE    RAWDATA(MAXLINLEN)
C
C     Read MAXLINLEN bytes starting from byte OFFSET and convert
C     them to characters. KCD lines are terminated by hex 0A (10).
C     Jump out of loop if the terminator is encountered.
C
      IF (READ_KCD_FILE(RAWDATA, OFFSET, (OFFSET+MAXLINLEN-1)) .NE. 0) 
     $     GOTO 942
      DO I = 1, MAXLINLEN
         IF (RAWDATA(I) .EQ. 10) THEN
            LINE(I:I) = ' '
            OFFSET = OFFSET + 1
            GOTO 941
         ELSE
            LINE(I:I) = CHAR(RAWDATA(I))
            OFFSET = OFFSET + 1
         ENDIF
      ENDDO
 941  LINELEN = I - 1
      READ_KCD_LINE = LINELEN
      RETURN
C
 942  READ_KCD_LINE = 0
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: PARSE_KCD_CHR
C
C     Description: Converts the value after the equals sign (=) in
C                  a KCD line into a string
C        
C**********************************************************************
C
      SUBROUTINE PARSE_KCD_CHR(CHRDATA, LINE, LINELEN)
      IMPLICIT NONE
C
C     Parameters
C
      CHARACTER*(*) CHRDATA
      CHARACTER*(*) LINE        ! The line to parse
      INTEGER       LINELEN     ! Number of characters in the line
C
C     Variables
C
      INTEGER EQUALS            ! The index of the equals sign
      INTEGER I
C
      EQUALS = INDEX(LINE(1:LINELEN), '=')
      DO I = 1, (LINELEN-EQUALS-1)
         CHRDATA(I:I) = LINE(EQUALS+1+I:EQUALS+1+I)
      ENDDO
      END
C
C
C
C**********************************************************************
C     Routine: PARSE_KCD_INT
C
C     Description: Converts the value after the equals sign (=) in
C                  a KCD line into an INTEGER
C        
C**********************************************************************
C
      INTEGER FUNCTION PARSE_KCD_INT(LINE, LINELEN)
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
C     Parameters
C
      CHARACTER*(*) LINE        ! The line to parse
      INTEGER       LINELEN     ! Number of characters in the line
C
C     Variables
C
      INTEGER EQUALS            ! The index of the equals sign
C
      EQUALS = INDEX(LINE(1:LINELEN), '=')
      READ(UNIT=LINE(EQUALS+2:LINELEN), FMT='(I6)') PARSE_KCD_INT
      RETURN
      END
C
C
C
C**********************************************************************
C     Routine: PARSE_KCD_REAL
C
C     Description: Converts the value after the equals sign (=) in
C                  a KCD line into a REAL
C
C**********************************************************************
C
      REAL FUNCTION PARSE_KCD_REAL(LINE, LINELEN)
      IMPLICIT NONE
      INCLUDE 'kcd.inc'
C
C     Parameters
C
      CHARACTER*(*) LINE        ! The line to parse
      INTEGER       LINELEN     ! Number of characters in the line
C
C     Variables
C
      INTEGER   EQUALS            ! The index of the equals sign
C
      EQUALS = INDEX(LINE(1:LINELEN), '=')
      READ(UNIT=LINE(EQUALS+2:LINELEN), FMT='(F20.6)') PARSE_KCD_REAL
      RETURN
      END
C
