C*********************************************************************
C*********************************************************************
C** 
C**     File: kcd.inc
C**
C**     Project: jitax_kcd
C**
C**     Description: Contains the commons used by the KCD file
C**                  reader routines in kcd.for
C**
C**
C*********************************************************************
C**
C** Copyright (c) 2003 Scott A. Belmonte
C**
C** E-mail: scott.belmonte@ntlworld.com
C**
C** This file is part of jitax_kcd. Reproduction, in whole or in part,
C** is only permitted under licence. For licensing information see
C** licence.txt at the top of the jitax_kcd source tree.
C**
C** E-mail: scott.belmonte@ntlworld.com
C**
C*********************************************************************
C*********************************************************************
C
      INTEGER REC_SIZE  ! The KCD file record size in bytes
      INTEGER UNT       ! The unit number that file is opened on
      INTEGER MAXCHRLEN ! Maximum string length
      INTEGER MAXLINLEN ! Maximum line length
      PARAMETER (REC_SIZE = 2048)
      PARAMETER (UNT=11)
      PARAMETER (MAXCHRLEN = 32)
      PARAMETER (MAXLINLEN = 100)
C
C     Common used by the binary file reading routines.
C
      BYTE    BUFFER(REC_SIZE) ! Read buffer for records
      INTEGER LAST_REC         ! The last record read into BUFFER
      LOGICAL BIG_END          ! .TRUE. if KCD file is big endian
      LOGICAL SWAP             ! If .TRUE. the bytes need to be
C                              ! before they can be used
      COMMON /KCD_FILE/ BUFFER, LAST_REC, BIG_END, SWAP
      SAVE /KCD_FILE/
C
C     Common containing the numeric frame information
C
      INTEGER DETID           ! Detector ID
      INTEGER GONID           ! Goniometer ID
      INTEGER SLOT            ! Slot number
      INTEGER REPEATS         ! Number of repeats
      INTEGER READOUTS        ! Number of readouts
      REAL    EXPTIME         ! Exposure time
      REAL    INTTIME         ! Integration time
      INTEGER WIDTH           ! X dimention
      INTEGER HEIGHT          ! Y dimention
      REAL    THETA           ! Theta start
      REAL    PHI             ! Phi start
      REAL    PHIRANGE        ! Phi scan range
      REAL    PHIINTERVAL     ! Phi scan interval
      REAL    OMEGA           ! Omega start
      REAL    KAPPA           ! Kappa start
      REAL    DXSTART         ! Dx start
      REAL    HVVOLT          ! HV voltage
      REAL    MACURRENT       ! MA current
      REAL    XSIZE           ! pixel X-size (um)
      REAL    YSIZE           ! pixel Y-size (um)
      REAL    ALPHA1          ! Alpha1
      REAL    ALPHA2          ! Alpha2
      REAL    ALPHRATIO       ! Alpha1/Alpha2 ratio
      REAL    KAPPAANG        ! Kappa-support angle
      INTEGER SCINTYPE        ! Scintillator type
      REAL    SCINLOAD        ! Scintillator load
      REAL    DETGAIN         ! Detector gain (ADU/count)
      REAL    BEAMDIA         ! Beam catcher diameter
      REAL    BEAMDIST        ! Beam catcher distance
      INTEGER BITSPERSAMPLE   ! Bits per pixel
      COMMON /KCD_INTS/ DETID, GONID, SLOT, REPEATS, READOUTS,
     $                  EXPTIME, INTTIME, WIDTH, HEIGHT, THETA,
     $                  PHI, PHIRANGE, PHIINTERVAL, OMEGA,
     $                  KAPPA, DXSTART, HVVOLT, MACURRENT,
     $                  XSIZE, YSIZE, ALPHA1, ALPHA2, ALPHRATIO,
     $                  KAPPAANG, SCINTYPE, SCINLOAD, DETGAIN,
     $                  BEAMDIA, BEAMDIST, BITSPERSAMPLE
      SAVE /KCD_INTS/
C
C     Common containing the frame string data
C
      CHARACTER ID(MAXCHRLEN)
      CHARACTER DATETIME(MAXCHRLEN)
      CHARACTER TARGET(MAXCHRLEN)
      CHARACTER POLARISATION(MAXCHRLEN)
      CHARACTER SHUTTER(MAXCHRLEN)
      CHARACTER DATTYPE(MAXCHRLEN)
      COMMON /KCD_CHARS/ ID, DATETIME, TARGET, POLARISATION,
     $                   SHUTTER, DATTYPE
      SAVE /KCD_CHARS/

